package net.sweevo.jinks;

import net.minecraft.ChatFormatting;
import net.minecraft.DefaultUncaughtExceptionHandler;
import net.minecraft.network.chat.Style;
import net.minecraft.server.MinecraftServer;
import net.sweevo.jinks.config.CommonConfig;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.annotation.Nonnull;
import java.io.File;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.Date;

public class BackupThread extends Thread {

    public static final Logger LOGGER = LoggerFactory.getLogger(BackupThread.class);
    private final MinecraftServer server;
    private final boolean quiet;
    private final File script;

    private BackupThread(@Nonnull MinecraftServer server, boolean quiet) {
        this.server = server;
        this.quiet = quiet;
        this.setName("ExtBackup");
        this.setUncaughtExceptionHandler(new DefaultUncaughtExceptionHandler(LOGGER));
        this.script = CommonConfig.getScript();
    }

    public static void tryCreateBackup(MinecraftServer server) {
        BackupData backupData = BackupData.get(server);
        if (BackupThread.shouldRunBackup(server)) {
            long currentTime = System.currentTimeMillis();
            BackupThread thread = new BackupThread(server, false);
            thread.start();
            backupData.updateSaveTime(currentTime);
        }
    }

    public static boolean shouldRunBackup(MinecraftServer server) {
        BackupData backupData = BackupData.get(server);
        if (!CommonConfig.isEnabled() || backupData.isPaused()) {
            return false;
        }
        return System.currentTimeMillis() - CommonConfig.getTimer() > backupData.getLastSaved();
    }

    public static void createBackup(MinecraftServer server, boolean quiet) {
        BackupData backupData = BackupData.get(server);
        long currentTime = System.currentTimeMillis();
        BackupThread thread = new BackupThread(server, quiet);
        thread.start();
        backupData.updateSaveTime(currentTime);
    }

    @Override
    public void run() {
        try {
            long start = System.currentTimeMillis();
            if (!this.quiet) { ChatHelper.broadcast(this.server,"ExtBackup started...", Style.EMPTY.withColor(ChatFormatting.BLUE)); }
            ExtBackup.LOGGER.info("Starting backup...");
            this.makeWorldBackup();
            long end = System.currentTimeMillis();
            String time = Timer.getTimer(end - start);
            if (!this.quiet) { ChatHelper.broadcast(this.server,"ExtBackup completed in %s", Style.EMPTY.withColor(ChatFormatting.BLUE), time); }
            ExtBackup.LOGGER.info("Backup done.");
        } catch (Exception e) {
            ExtBackup.LOGGER.error("Error backing up", e);
        }
    }

    // vanilla copy with modifications
    @SuppressWarnings("CallToPrintStackTrace")
    private void makeWorldBackup() {
        if (!script.exists() || !script.canExecute()) {
            ExtBackup.LOGGER.error("Cannot access or execute backup script. Bailing out!");
        }

        ProcessBuilder pb = new ProcessBuilder(script.getAbsolutePath());
        int returnValue;
        pb.redirectErrorStream(true);
        try {
            Process backup = pb.start();
            returnValue = backup.waitFor();
            if (returnValue != 0) {
                throw new IOException("Backup process returned non-zero result!");
            }
        } catch (Exception ex) {
            ExtBackup.LOGGER.error("Something went wrong with the Backup script!");
            ExtBackup.LOGGER.error("Check your Backups.");
            ex.printStackTrace();
        }
    }

    private static class Timer {

        private static final SimpleDateFormat SECONDS = new SimpleDateFormat("s.SSS");
        private static final SimpleDateFormat MINUTES = new SimpleDateFormat("mm:ss");
        private static final SimpleDateFormat HOURS = new SimpleDateFormat("HH:mm");

        public static String getTimer(long milliseconds) {
            Date date = new Date(milliseconds);
            double seconds = milliseconds / 1000d;
            if (seconds < 60) {
                return SECONDS.format(date) + "s";
            } else if (seconds < 3600) {
                return MINUTES.format(date) + "min";
            } else {
                return HOURS.format(date) + "h";
            }
        }
    }
}
